# -*- coding: UTF-8 -*-
# mod Nikolasi for TMBD plugin
PLUGIN_PATH = '/usr/lib/enigma2/python/Plugins/Extensions/TMBD'
from GUIComponent import GUIComponent
from Tools.FuzzyDate import FuzzyTime
from ServiceReference import ServiceReference
from Components.MultiContent import MultiContentEntryText, MultiContentEntryPixmapAlphaTest, MultiContentEntryProgress
from Components.config import config
import os
import struct
import random
from Tools.LoadPixmap import LoadPixmap
from Tools.Directories import fileExists, resolveFilename, SCOPE_PLUGINS, SCOPE_SKIN_IMAGE, SCOPE_CURRENT_SKIN
from Screens.LocationBox import defaultInhibitDirs
import NavigationInstance
import skin
from enigma import eListboxPythonMultiContent, eListbox, gFont, iServiceInformation, RT_HALIGN_LEFT, RT_HALIGN_RIGHT, eServiceReference, eServiceCenter, eTimer, ePicLoad, loadPNG
from enigma import addFont
try:
    addFont('%s/nmsbdmovie.ttf' % PLUGIN_PATH, 'RegularMovie', 95, 1)
    addFont('%s/nmsbdmovie2.ttf' % PLUGIN_PATH, 'RegularMovie2', 95, 1)
except Exception as ex:
    print ex
AUDIO_EXTENSIONS = frozenset(('.dts',
 '.mp3',
 '.wav',
 '.wave',
 '.ogg',
 '.flac',
 '.m4a',
 '.mp2',
 '.m2a',
 '.3gp',
 '.3g2',
 '.asf',
 '.wma'))
DVD_EXTENSIONS = ('.iso', '.img')
IMAGE_EXTENSIONS = frozenset(('.jpg',
 '.png',
 '.gif',
 '.bmp'))
MOVIE_EXTENSIONS = frozenset(('.mpg',
 '.vob',
 '.wav',
 '.m4v',
 '.mkv',
 '.avi',
 '.divx',
 '.dat',
 '.flv',
 '.mp4',
 '.mov',
 '.wmv',
 '.m2ts'))
KNOWN_EXTENSIONS = MOVIE_EXTENSIONS.union(IMAGE_EXTENSIONS, DVD_EXTENSIONS, AUDIO_EXTENSIONS)
cutsParser = struct.Struct('>QI')

MEDIAEXTENSIONS = {
                "ts": "movie",
                "iso": "dvd",
                "img": "dvd",
		"m4a": "music",
		"mp2": "music",
		"mp3": "music",
		"wav": "music",
		"ogg": "music",
		"flac": "music",
		"gif": "picture",                
		"jpg": "picture",
		"jpeg": "picture",
		"png": "picture",
		"bmp": "picture",
		"avi": "movie",
		"divx": "movie",
		"mpg": "movie",
		"mpeg": "movie",
		"mkv": "movie",
		"mp4": "movie",
                "m2ts": "movie",
                "flv": "movie",                
		"mov": "movie"
	}
MEDIAEXTENSIONS2 = {
                "iso": "iso",
                "img": "iso",
		"m4a": "music",
		"mp2": "music",
		"mp3": "music",
		"wav": "music",
		"ogg": "music",
		"flac": "music",
		"gif": "picture",                
		"jpg": "picture",
		"jpeg": "picture",
		"png": "picture",
		"bmp": "picture",
		"avi": "movie",
		"divx": "movie",
		"mpg": "movie",
		"mpeg": "movie",
		"mkv": "movie",
		"mp4": "movie",
                "m2ts": "movie",
                "flv": "movie",                
		"mov": "movie"
	}

def checkIcon(filename):
    path = '/media/hdd/covers/'
    if filename == ".Trash":
        local = "/usr/lib/enigma2/python/Plugins/Extensions/TMBD/corzina.jpg"
    elif filename == ".Corndir":
        local = "/usr/lib/enigma2/python/Plugins/Extensions/TMBD/back.jpg"            
    else:    
	local = path + filename + '.jpg'
    if fileExists(local):
	    localfile = local 
    else:
            localfile = "/usr/lib/enigma2/python/Plugins/Extensions/TMBD/no_poster2.jpg"
    return localfile

def pixmap(filename):
    picon = ePicLoad()
    myicon = checkIcon(filename)
    piconWidth = 62
    piconHeight = 92
    picon.setPara((piconWidth, piconHeight, 1, 1, False, 1, '#ffffffff'))
    picon.startDecode(myicon, 0, 0, False)
    pixmap = picon.getData()
    return pixmap
	    
class MovieListData():
    pass


class StubInfo():

    def getName(self, serviceref):
        return os.path.split(serviceref.getPath())[1]

    def getLength(self, serviceref):
        return -1

    def getEvent(self, serviceref, *args):
        return None

    def isPlayable(self):
        return True

    def getInfo(self, serviceref, w):
        if w == iServiceInformation.sTimeCreate:
            return os.stat(serviceref.getPath()).st_ctime
        if w == iServiceInformation.sFileSize:
            return os.stat(serviceref.getPath()).st_size
        if w == iServiceInformation.sDescription:
            return serviceref.getPath()
        return 0

    def getInfoString(self, serviceref, w):
        return ''


justStubInfo = StubInfo()

def lastPlayPosFromCache(ref):
    from Screens.InfoBarGenerics import resumePointCache
    return resumePointCache.get(ref.toString(), None)


def moviePlayState(cutsFileName, ref, length):
    try:
        f = open(cutsFileName, 'rb')
        lastCut = None
        cutPTS = None
        while 1:
            data = f.read(cutsParser.size)
            if len(data) < cutsParser.size:
                break
            cut, cutType = cutsParser.unpack(data)
            if cutType == 3:
                cutPTS = cut
            else:
                lastCut = cut

        f.close()
        last = lastPlayPosFromCache(ref)
        if last:
            if not lastCut:
                lastCut = last[2]
            if not cutPTS:
                cutPTS = last[1]
        if cutPTS is None:
            return
        if not lastCut:
            if length and length > 0:
                lastCut = length * 90000
            else:
                return 50
        if cutPTS >= lastCut:
            return 100
        return 100 * cutPTS // lastCut
    except:
        cutPTS = lastPlayPosFromCache(ref)
        if cutPTS:
            if not length or length < 0:
                length = cutPTS[2]
            if length:
                if cutPTS[1] >= length:
                    return 100
                return 100 * cutPTS[1] // length
            else:
                return 50
        return

    return


def resetMoviePlayState(cutsFileName, ref = None):
    try:
        if ref is not None:
            from Screens.InfoBarGenerics import delResumePoint
            delResumePoint(ref)
        f = open(cutsFileName, 'rb')
        cutlist = []
        while 1:
            data = f.read(cutsParser.size)
            if len(data) < cutsParser.size:
                break
            cut, cutType = cutsParser.unpack(data)
            if cutType != 3:
                cutlist.append(data)

        f.close()
        f = open(cutsFileName, 'wb')
        f.write(''.join(cutlist))
        f.close()
    except:
        pass

    return


class MovieList(GUIComponent):
    SORT_ALPHANUMERIC = 1
    SORT_RECORDED = 2
    SHUFFLE = 3
    SORT_ALPHANUMERIC_REVERSE = 4
    SORT_RECORDED_REVERSE = 5
    SORT_ALPHANUMERIC_FLAT = 6
    SORT_ALPHANUMERIC_FLAT_REVERSE = 7
    LISTTYPE_ORIGINAL = 1
    LISTTYPE_COMPACT_DESCRIPTION = 2
    LISTTYPE_COMPACT = 3
    LISTTYPE_MINIMAL = 4
    LISTTYPE_ORIGINAL2 = 5
    LISTTYPE_MINIMAL2 = 6
    LISTTYPE_MINIMAL_PROGRES = 7
    HIDE_DESCRIPTION = 1
    SHOW_DESCRIPTION = 2

    def __init__(self, root, list_type = None, sort_type = None, descr_state = None):
        GUIComponent.__init__(self)
        self.list = []
        self.list_type = list_type or self.LISTTYPE_ORIGINAL2
        self.descr_state = descr_state or self.HIDE_DESCRIPTION
        self.sort_type = sort_type or self.SORT_RECORDED
        self.firstFileEntry = 0
        self.parentDirectory = 0
        self.fontName2 = 'Regular'
        if config.movielist.MovieSelectionfonts.value == "0":
             self.fontName = self.fontName2
        if config.movielist.MovieSelectionfonts.value == "1":
             self.fontName = 'RegularMovie'
        if config.movielist.MovieSelectionfonts.value == "2":
             self.fontName = 'RegularMovie2'              
        self.fontSizesOriginal = (22, 18, 16)
        self.fontSizesCompact = (20, 14)
        self.fontSizesMinimal = (20, 16)
        self.itemHeights = (75, 37, 25)
        self.reloadDelayTimer = None
        self.l = eListboxPythonMultiContent()
        self.tags = set()
        self.root = None
        self._playInBackground = None
        self._char = ''
        if root is not None:
            self.reload(root)
        self.l.setBuildFunc(self.buildMovieListEntry)
        self.onSelectionChanged = []
        self.iconPart = []
        for part in range(5):
            self.iconPart.append(LoadPixmap(resolveFilename(SCOPE_SKIN_IMAGE, 'skin_default/icons/part_%d_4.png' % part)))
        self.iconMovieRec = LoadPixmap(resolveFilename(SCOPE_SKIN_IMAGE, 'skin_default/icons/part_new.png'))
        self.iconMoviePlay = LoadPixmap(resolveFilename(SCOPE_SKIN_IMAGE, 'skin_default/icons/movie_play.png'))
        self.iconMoviePlayRec = LoadPixmap(resolveFilename(SCOPE_SKIN_IMAGE, 'skin_default/icons/movie_play_rec.png'))
        self.iconUnwatched = LoadPixmap(resolveFilename(SCOPE_SKIN_IMAGE, 'skin_default/icons/part_unwatched.png'))
        self.iconFolder = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/TMBD/ico/directory.png")
        self.iconTrash = LoadPixmap(resolveFilename(SCOPE_SKIN_IMAGE, 'skin_default/icons/trashcan.png'))
        self.runningTimers = {}
        self.updateRecordings()
        return

    def get_playInBackground(self):
        return self._playInBackground

    def set_playInBackground(self, value):
        self._playInBackground = value
        self.reload()

    playInBackground = property(get_playInBackground, set_playInBackground)

    def updateRecordings(self, timer = None):
        if timer is not None:
            if timer.justplay:
                return
        result = {}
        for timer in NavigationInstance.instance.RecordTimer.timer_list:
            if timer.isRunning() and not timer.justplay:
                result[os.path.split(timer.Filename)[1] + '.ts'] = timer
        if self.runningTimers == result:
            return
        else:
            self.runningTimers = result
            if timer is not None:
                if self.reloadDelayTimer is not None:
                    self.reloadDelayTimer.stop()
                self.reloadDelayTimer = eTimer()
                self.reloadDelayTimer.callback.append(self.reload)
                self.reloadDelayTimer.start(5000, 1)
            return

    def connectSelChanged(self, fnc):
        if fnc not in self.onSelectionChanged:
            self.onSelectionChanged.append(fnc)

    def disconnectSelChanged(self, fnc):
        if fnc in self.onSelectionChanged:
            self.onSelectionChanged.remove(fnc)

    def selectionChanged(self):
        for x in self.onSelectionChanged:
            x()

    def setListType(self, type):
        if type != self.list_type:
            self.list_type = type
            self.redrawList()

    def setDescriptionState(self, val):
        self.descr_state = val

    def setSortType(self, type):
        self.sort_type = type

    def applySkin(self, desktop, parent):
        attribs = []
        if self.skinAttributes is not None:
            attribs = []
            for attrib, value in self.skinAttributes:
                try:
                    if attrib == 'fontName':
                        self.fontName2 = value
                    elif attrib == 'fontSizesOriginal':
                        self.fontSizesOriginal = map(int, value.split(','))
                    elif attrib == 'fontSizesCompact':
                        self.fontSizesCompact = map(int, value.split(','))
                    elif attrib == 'fontSizesMinimal':
                        self.fontSizesMinimal = map(int, value.split(','))
                    elif attrib == 'itemHeights':
                        self.itemHeights = map(int, value.split(','))
                    else:
                        attribs.append((attrib, value))
                except Exception as e:
                    print '[MovieList] Error "%s" parsing attribute: %s="%s"' % (str(e), attrib, value)

        self.skinAttributes = attribs
        self.redrawList()
        return GUIComponent.applySkin(self, desktop, parent)

    def redrawList(self):
        if self.list_type == MovieList.LISTTYPE_ORIGINAL:
            for i in range(3):
                self.l.setFont(i, gFont(self.fontName, self.fontSizesOriginal[i]))
            self.itemHeight = self.itemHeights[0]
        elif self.list_type == MovieList.LISTTYPE_ORIGINAL2:
            for i in range(3):
                self.l.setFont(i, gFont(self.fontName, self.fontSizesOriginal[i]))
            self.itemHeight = 117            
        elif self.list_type == MovieList.LISTTYPE_COMPACT_DESCRIPTION or self.list_type == MovieList.LISTTYPE_COMPACT:
            for i in range(2):
                self.l.setFont(i, gFont(self.fontName, self.fontSizesCompact[i]))
            self.itemHeight = self.itemHeights[1]
        else:
            for i in range(2):
                self.l.setFont(i, gFont(self.fontName, self.fontSizesMinimal[i]))
            self.itemHeight = self.itemHeights[2]
        self.l.setItemHeight(self.itemHeight)

    def invalidateItem(self, index):
        x = self.list[index]
        self.list[index] = (x[0],
         x[1],
         x[2],
         None)
        return

    def invalidateCurrentItem(self):
        self.invalidateItem(self.getCurrentIndex())

    def buildMovieListEntry(self, serviceref, info, begin, data):
        txt2 = ""
        txt = ""
        text = ""
        name = ""
        Country = ''
        Country2 = ''
        genre = ''
        descript = ''
        ratingall = ''
        actors = ''
        namedesc = ''
        filename = ""
        description = ''
        year = ''
        year2 = ''                
        director = ''
        director2 = ''
        dir = ''
        namestars = ""
        starss = ""
        desc = ""
        namedir = ""
        nameext = ""
        altext = ""
        namefil = ""
        namepng = ""
        altname = ""
        nametxt = ""
        nams = ""
        counttochra = 1
        textfoldersTrash = _('Trash can')
        textfolders = _('Directory')
        width = self.l.getItemSize().width()
        pathName = serviceref.getPath()
        res = [None]
        if serviceref.flags & eServiceReference.mustDescent:
            if self.list_type == MovieList.LISTTYPE_ORIGINAL2:
                #text = serviceref.getName()
                number = '%s' % (os.path.dirname(pathName))
                if number.endswith("VIDEO_TS"):
                    number = number[:-9]
                if info is None:
                    txt = ""
                    text = '..'
                    if config.movielist.folder.value == "0":
                        txt = ".Corndir"
                    elif config.movielist.folder.value == "1":
                        conf = config.movielist.last_videodir.value
                        number = '%s' % (os.path.dirname(conf))
                        txt = ".Corndir"
                    elif config.movielist.folder.value == "2":                        
                        conf = config.movielist.last_videodir.value
                        number = '%s' % (os.path.dirname(conf))
	                if number != "":
	                    p = os.path.split(number)
	                    if not p[1]:
		                p = os.path.split(p[0])
                            txt = '%s' % (p[1])
                        path = '/media/hdd/covers/'
                        local = path + txt + '.jpg'
                        if fileExists(local):
                            txt = txt
                        else:
                            txt = ".Corndir"
                else:
                    p = os.path.split(pathName)
                    if not p[1]:
                        p = os.path.split(p[0])
                    text = p[1]
                if text == 'VIDEO_TS':
	             if number != "":
	                p = os.path.split(number)
	                if not p[1]:
		            p = os.path.split(p[0])
                        txt2 = '%s' % (p[1])
                elif text == '..':
                        txt2 = txt                    
                else:
                        txt2 = '%s' % (text)
                if config.plugins.tmbd.movieselectionparts.value == "none":
                    namedir = number
                else:
                    namedir = config.plugins.tmbd.movieselectionparts.value + "/" + txt2
                if fileExists("%s.txt" %(namedir)):   
                    readmetafile = open("%s.txt" %(namedir), "r")
		    Country2 = readmetafile.readline()[0:-1]
		    if "Страна:" in Country2:
                        Country = Country2.split('Страна:')[1].strip()
                    else:
                        Country = ""
		    genre = readmetafile.readline()[0:-1]
		    descript = readmetafile.readline()[0:-1]
		    ratingall = readmetafile.readline()[0:-1]
		    if 'Рейтинг:' in ratingall:
			    namestars = ratingall.split('Рейтинг:')[1].split('.')[0].strip()
			    starss = resolveFilename(SCOPE_PLUGINS, "Extensions/TMBD/starsbar/%s.png") % (namestars)
		    actors = readmetafile.readline()[0:-1]
		    director = readmetafile.readline()[0:-1]
		    if "Режиссер:" in director:
                        director2 = director.split('Режиссер:')[1].strip()
                    else:
                        director2 = ""			            
		    year = readmetafile.readline()[0:-1]
		    if "Год выпуска:" in year:
                        year2 = year.split('Год выпуска:')[1].strip()
                    else:
                        year2 = ""
                    nams = readmetafile.readline()[0:-1]
		    if "Origname:" in nams:
                        nams = nams.split('Origname:')[1].strip()
                    else:
                        nams = ""                          
                    readmetafile.close()
                    dir = '%s, реж. %s, %s год' % (Country, director2, year2)
                    if text == '..' and config.movielist.folder.value == "0":
                        desc = 'Переход в каталог %s' % (number)
                    else:    
                        desc = '%s\n%s\n%s' % (genre, dir, ratingall)                                    
                else:
                    if text == '..':
                        desc = 'Переход в каталог %s' % (number)
                    else:    
                        desc = 'Нет данных'
                if config.movielist.profilcolor.value == "None":
                    colors = None
                else:    
                    colors = int(config.movielist.profilcolor.value)
		if config.movielist.originalname.value:
                    if nams != "":
                        nametxt = nams
                    else:
                        nametxt = text 
                else:
                    nametxt = text                    
                ramka = resolveFilename(SCOPE_PLUGINS, "Extensions/TMBD/ramka.png")    
            	res = [ None ]
            	res.append(MultiContentEntryPixmapAlphaTest(pos=(0, 12), size=(62, 92), png=loadPNG(ramka)))
                png = LoadPixmap(cached=True, path="/usr/lib/enigma2/python/Plugins/Extensions/TMBD/extensions/directory.png")
     		res.append((eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, 70, 2, 20, 20, png))
		res.append(MultiContentEntryText(pos=(95, 0), size=(width-40, 30), font = 0, flags = RT_HALIGN_LEFT, text=nametxt, color=colors))
                res.append(MultiContentEntryText(pos=(70, 24), size=(width-40, 65), font = 1, flags = RT_HALIGN_LEFT, text=desc))
                res.append(MultiContentEntryPixmapAlphaTest(pos=(3, 14), size=(56, 86), png=pixmap(txt2)))
                if fileExists(starss):
                        res.append(MultiContentEntryPixmapAlphaTest(pos=(70, 90), size=(210, 21), png=loadPNG(starss)))
		return res
	    else:
                if self.list_type in (MovieList.LISTTYPE_MINIMAL2, MovieList.LISTTYPE_MINIMAL_PROGRES):
                   textfoldersTrash = ''
                   textfolders = ''
                iconSize = 22
                if info is None:
                    txt = '..'
                    self.iconFolder = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/TMBD/ico/back.png")
                    res.append(MultiContentEntryPixmapAlphaTest(pos=(0, 1), size=(iconSize, iconSize), png=self.iconFolder))
                    res.append(MultiContentEntryText(pos=(iconSize + 2, 0), size=(width - 166, self.itemHeight), font=0, flags=RT_HALIGN_LEFT, text=txt))
                    res.append(MultiContentEntryText(pos=(width - 145, 4), size=(145, self.itemHeight), font=1, flags=RT_HALIGN_RIGHT, text=textfolders))
                    return res                    
                else:
                    p = os.path.split(pathName)
                    if not p[1]:
                        p = os.path.split(p[0])
                    txt = p[1]
                    if txt == '.Trash':
                        res.append(MultiContentEntryPixmapAlphaTest(pos=(0, 1), size=(iconSize, 24), png=self.iconTrash))
                        res.append(MultiContentEntryText(pos=(iconSize + 2, 0), size=(width - 166, self.itemHeight), font=0, flags=RT_HALIGN_LEFT, text=_('Deleted items')))
                        res.append(MultiContentEntryText(pos=(width - 145, 4), size=(145, self.itemHeight), font=1, flags=RT_HALIGN_RIGHT, text=textfoldersTrash))
                        return res
    		    if config.movielist.originalname.value:
                        filename = '%s' % (os.path.dirname(pathName))
                        nams = self.getarealname(filename)  		
                        if nams != "":
                            nametxt = nams
                        else:
                            nametxt = txt 
                    else:
                        nametxt = txt                       
                    self.iconFolder = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/TMBD/ico/directory.png")
                    res.append(MultiContentEntryPixmapAlphaTest(pos=(0, 1), size=(iconSize, iconSize), png=self.iconFolder))
                    res.append(MultiContentEntryText(pos=(iconSize + 2, 0), size=(width - 166, self.itemHeight), font=0, flags=RT_HALIGN_LEFT, text=txt))
                    res.append(MultiContentEntryText(pos=(width - 145, 4), size=(145, self.itemHeight), font=1, flags=RT_HALIGN_RIGHT, text=textfolders))
                    return res
        else:
            if data == -1 or data is None:
                data = MovieListData()
                cur_idx = self.l.getCurrentSelectionIndex()
                x = self.list[cur_idx]
                if config.usage.load_length_of_movies_in_moviellist.value:
                    data.len = x[1].getLength(x[0])
                else:
                    data.len = 0
                self.list[cur_idx] = (x[0],
                 x[1],
                 x[2],
                 data)
                data.txt = info.getName(serviceref)
                if config.movielist.hide_extensions.value:
                    fileName, fileExtension = os.path.splitext(data.txt)
                    if fileExtension in KNOWN_EXTENSIONS:
                        data.txt = fileName
                data.icon = None
                data.part = None
                if os.path.split(pathName)[1] in self.runningTimers:
                    if self.playInBackground and serviceref == self.playInBackground:
                        data.icon = self.iconMoviePlayRec
                    else:
                        data.icon = self.iconMovieRec
                elif self.playInBackground and serviceref == self.playInBackground:
                    data.icon = self.iconMoviePlay
                else:
                    switch = config.usage.show_icons_in_movielist.value
                    data.part = moviePlayState(pathName + '.cuts', serviceref, data.len)
                    if switch == 'i':
                        if data.part is None:
                            if config.usage.movielist_unseen.value:
                                data.icon = self.iconUnwatched
                        else:
                            data.icon = self.iconPart[data.part // 25]
                    elif switch == 'p' or switch == 's':
                        if data.part is None:
                            if config.usage.movielist_unseen.value:
                                data.part = 0
                            data.partcol = 8421504
                        else:
                            data.partcol = 15790320
                service = ServiceReference(info.getInfoString(serviceref, iServiceInformation.sServiceref))
                if service is None:
                    data.serviceName = None
                else:
                    data.serviceName = service.getServiceName()
                data.description = info.getInfoString(serviceref, iServiceInformation.sDescription)
            len = data.len
            if len > 0:
                len = '%d:%02d' % (len / 60, len % 60)
            else:
                len = ''
            if data.icon is not None:
                iconSize = 22
                if self.list_type in (MovieList.LISTTYPE_COMPACT_DESCRIPTION, MovieList.LISTTYPE_COMPACT):
                    pos = (0, 0)
                else:
                    pos = (0, 1)
                if self.list_type in (MovieList.LISTTYPE_ORIGINAL2, MovieList.LISTTYPE_MINIMAL2, MovieList.LISTTYPE_MINIMAL_PROGRES):
                    pass
                else:
                    res.append(MultiContentEntryPixmapAlphaTest(pos=pos, size=(iconSize, 20), png=data.icon))
            switch = config.usage.show_icons_in_movielist.value
            if switch == 'p' or switch == 's':
                if switch == 'p':
                    iconSize = 48
                else:
                    iconSize = 22
                if data.part is not None:
                    if self.list_type in (MovieList.LISTTYPE_ORIGINAL2, MovieList.LISTTYPE_MINIMAL2, MovieList.LISTTYPE_MINIMAL_PROGRES):
                        pass
                    else:
                        res.append(MultiContentEntryProgress(pos=(0, 5), size=(iconSize - 2, 16), percent=data.part, borderWidth=2, foreColor=data.partcol, foreColorSelected=None, backColor=None, backColorSelected=None))
            elif switch == 'i':
                iconSize = 22
            else:
                iconSize = 0
            begin_string = ''
            if begin > 0:
                begin_string = ', '.join(FuzzyTime(begin, inPast=True))
            ih = self.itemHeight
            if self.list_type == MovieList.LISTTYPE_ORIGINAL:
 		if config.movielist.originalname.value:
                    filename = os.path.realpath(pathName)
                    nams = self.getarealname(filename)  		
                    if nams != "":
                            nametxt = nams
                    else:
                            nametxt = data.txt 
                else:
                    nametxt = data.txt                
                ih1 = ih * 2 / 5
                ih2 = ih * 2 / 3
                res.append(MultiContentEntryText(pos=(iconSize, 0), size=(width - 182, ih1), font=0, flags=RT_HALIGN_LEFT, text=nametxt))
                if self.tags:
                    res.append(MultiContentEntryText(pos=(width - 180, 0), size=(180, ih1), font=2, flags=RT_HALIGN_RIGHT, text=info.getInfoString(serviceref, iServiceInformation.sTags)))
                    if data.serviceName:
                        res.append(MultiContentEntryText(pos=(200, ih2), size=(200, ih2 - ih1), font=1, flags=RT_HALIGN_LEFT, text=data.serviceName))
                elif data.serviceName:
                    res.append(MultiContentEntryText(pos=(width - 180, 0), size=(180, ih1), font=2, flags=RT_HALIGN_RIGHT, text=data.serviceName))
                res.append(MultiContentEntryText(pos=(0, ih1), size=(width, ih2 - ih1), font=1, flags=RT_HALIGN_LEFT, text=data.description))
                res.append(MultiContentEntryText(pos=(0, ih2), size=(200, ih - ih2), font=1, flags=RT_HALIGN_LEFT, text=begin_string))
                if len:
                    res.append(MultiContentEntryText(pos=(width - 200, ih2), size=(198, ih - ih2), font=1, flags=RT_HALIGN_RIGHT, text=len))
            if self.list_type == MovieList.LISTTYPE_ORIGINAL2:
                        filename = os.path.realpath(pathName)
	                if filename != "":
                                altname, name = os.path.split(filename) or ''
                                if name.endswith('.ts'):
                                    name = data.txt
                                counttochra = int(name.count("."))
                                if "_" in name:
                                    name = name.replace("_", " ")
                                if counttochra > 1:
                                    counttochra = counttochra - 1
                                    name = name.replace(".", " ", counttochra)                                     
                                path = '/media/hdd/covers/'
                                if fileExists(path + name + '.jpg'):
                                    namepng = name
                                else:
                                    namepng = self.getalttext(altname)                                    
                        if config.plugins.tmbd.movieselectionparts.value == "none":
                            nameext = filename[:-2] + 'txt'
                            if fileExists(nameext):
                                namedesc = nameext
                            else:
                                if filename != "":
                                    namedesc = altname + '.txt'
                                    namefil = self.getalttext(altname)
                        else:
                            altext, namedir = os.path.split(filename)
                            nameext = config.plugins.tmbd.movieselectionparts.value + "/" + namedir[:-2] + 'txt'
                            if fileExists(nameext):
                                namedesc = nameext
                            else:
                                namefil = self.getalttext(altext)
                                namedesc = config.plugins.tmbd.movieselectionparts.value + "/" + namefil + '.txt'
                        if fileExists(namedesc):   
                            readmetafile = open(namedesc, "r")
			    Country2 = readmetafile.readline()[0:-1]
			    if "Страна:" in Country2:
                                Country = Country2.split('Страна:')[1].strip()
                            else:
                                Country = ""
			    genre = readmetafile.readline()[0:-1]
			    descript = readmetafile.readline()[0:-1]
			    ratingall = readmetafile.readline()[0:-1]
			    if 'Рейтинг:' in ratingall:
			            namestars = ratingall.split('Рейтинг:')[1].split('.')[0].strip()
			            starss = resolveFilename(SCOPE_PLUGINS, "Extensions/TMBD/starsbar/%s.png") % (namestars)
			    actors = readmetafile.readline()[0:-1]
			    director = readmetafile.readline()[0:-1]
			    if "Режиссер:" in director:
                                director2 = director.split('Режиссер:')[1].strip()
                            else:
                                director2 = ""			            
			    year = readmetafile.readline()[0:-1]
			    if "Год выпуска:" in year:
                                year2 = year.split('Год выпуска:')[1].strip()
                            else:
                                year2 = ""
                            nams = readmetafile.readline()[0:-1]
			    if "Origname:" in nams:
                                nams = nams.split('Origname:')[1].strip()
                            else:
                                nams = ""                            
                            readmetafile.close()
                            dir = '%s, реж. %s, %s год' % (Country, director2, year2)
                            desc = '%s\n%s\n%s' % (genre, dir, ratingall)                                    
                        else:
                            desc = 'Нет данных'
                        if config.movielist.profilcolor.value == "None":
                            colors = None
                        else:    
                            colors = int(config.movielist.profilcolor.value)                            
			extension = filename
			extension = extension.split('.')
			extension = extension[-1].lower()
			offset = 25
			if MEDIAEXTENSIONS.has_key(extension):
				png = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/TMBD/extensions/" + MEDIAEXTENSIONS[extension] + ".png")
			else:
				png = None
			if config.movielist.originalname.value:
                            if nams != "":
                                nametxt = nams
                            else:
                                nametxt = data.txt 
                        else:
                            nametxt = data.txt
                        ramka = resolveFilename(SCOPE_PLUGINS, "Extensions/TMBD/ramka.png")
                        res.append(MultiContentEntryPixmapAlphaTest(pos=(0, 12), size=(62, 92), png=loadPNG(ramka)))
			res.append((eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, 70, 2, 20, 20, png))
			res.append(MultiContentEntryText(pos=(0+offset+70, 0), size=(width-182, 30), font = 0, flags = RT_HALIGN_LEFT, text=nametxt, color=colors))
                        res.append(MultiContentEntryText(pos=(70, 24), size=(width-40, 65), font = 1, flags = RT_HALIGN_LEFT, text=desc))
                        res.append(MultiContentEntryPixmapAlphaTest(pos=(3, 14), size=(56, 86), png=pixmap(namepng)))
                        if fileExists(starss):
                                res.append(MultiContentEntryPixmapAlphaTest(pos=(70, 90), size=(210, 21), png=loadPNG(starss)))
            elif self.list_type == MovieList.LISTTYPE_COMPACT_DESCRIPTION:
    		if config.movielist.originalname.value:
                    filename = os.path.realpath(pathName)
                    nams = self.getarealname(filename)  		
                    if nams != "":
                            nametxt = nams
                    else:
                            nametxt = data.txt 
                else:
                    nametxt = data.txt                 
                ih1 = (ih * 8 + 14) / 15
                if len:
                    lenSize = 58 * ih / 37
                else:
                    lenSize = 0
                res.append(MultiContentEntryText(pos=(iconSize, 0), size=(width - 140, ih1), font=0, flags=RT_HALIGN_LEFT, text=nametxt))
                res.append(MultiContentEntryText(pos=(0, ih1), size=(width - 154 - lenSize, ih - ih1), font=1, flags=RT_HALIGN_LEFT, text=data.description))
                res.append(MultiContentEntryText(pos=(width - 120, 6), size=(120, ih1), font=1, flags=RT_HALIGN_RIGHT, text=begin_string))
                if data.serviceName:
                    res.append(MultiContentEntryText(pos=(width - 154 - lenSize, ih1), size=(154, ih - ih1), font=1, flags=RT_HALIGN_RIGHT, text=data.serviceName))
                if lenSize:
                    res.append(MultiContentEntryText(pos=(width - lenSize, ih1), size=(lenSize, ih - ih1), font=1, flags=RT_HALIGN_RIGHT, text=len))
            elif self.list_type == MovieList.LISTTYPE_COMPACT:
    		if config.movielist.originalname.value:
                    filename = os.path.realpath(pathName)
                    nams = self.getarealname(filename)  		
                    if nams != "":
                            nametxt = nams
                    else:
                            nametxt = data.txt 
                else:
                    nametxt = data.txt                 
                ih1 = (ih * 8 + 14) / 15
                if len:
                    lenSize = 2 * ih
                else:
                    lenSize = 0
                res.append(MultiContentEntryText(pos=(iconSize, 0), size=(width - lenSize - iconSize, ih1), font=0, flags=RT_HALIGN_LEFT, text=nametxt))
                if self.tags:
                    res.append(MultiContentEntryText(pos=(width - 200, ih1), size=(200, ih - ih1), font=1, flags=RT_HALIGN_RIGHT, text=info.getInfoString(serviceref, iServiceInformation.sTags)))
                    if data.serviceName:
                        res.append(MultiContentEntryText(pos=(200, ih1), size=(200, ih - ih1), font=1, flags=RT_HALIGN_LEFT, text=data.serviceName))
                elif data.serviceName:
                    res.append(MultiContentEntryText(pos=(width - 200, ih1), size=(200, ih - ih1), font=1, flags=RT_HALIGN_RIGHT, text=data.serviceName))
                res.append(MultiContentEntryText(pos=(0, ih1), size=(200, ih - ih1), font=1, flags=RT_HALIGN_LEFT, text=begin_string))
                if lenSize:
                    res.append(MultiContentEntryText(pos=(width - lenSize, 0), size=(lenSize, ih1), font=0, flags=RT_HALIGN_RIGHT, text=len))
            elif self.list_type == MovieList.LISTTYPE_MINIMAL2:
                filename = os.path.realpath(pathName)
		extension = filename
		extension = extension.split('.')
		extension = extension[-1].lower()
		if config.movielist.originalname.value:
                        nams = self.getarealname(filename)  		
                        if nams != "":
                              nametxt = nams
                        else:
                             nametxt = data.txt 
                else:
                    nametxt = data.txt		
		if MEDIAEXTENSIONS2.has_key(extension):
			png = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/TMBD/ico/" + MEDIAEXTENSIONS[extension] + ".png")
		else:
                        if data.part is None:
                             png = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/TMBD/ico/new.png")
                        else:
                             png = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/TMBD/ico/wathc.png")
                res.append((eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, 0, 1, 20, 20, png))
                res.append(MultiContentEntryText(pos=(25, 0), size=(width - 30, ih), font=0, flags=RT_HALIGN_LEFT, text=nametxt))
            elif self.list_type == MovieList.LISTTYPE_MINIMAL_PROGRES:
                if data.part is None:
                    dataext = 0
                    partcol = 8421504
                else:
                    dataext = data.part
                    partcol = 15790320                
                filename = os.path.realpath(pathName)
		if config.movielist.originalname.value:
                        nams = self.getarealname(filename)  		
                        if nams != "":
                              nametxt = nams
                        else:
                             nametxt = data.txt 
                else:
                    nametxt = data.txt	                
		extension = filename
		extension = extension.split('.')
		extension = extension[-1].lower()
		if MEDIAEXTENSIONS2.has_key(extension):
			png = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/TMBD/ico/" + MEDIAEXTENSIONS[extension] + ".png")
		else:
                        if data.part is None:
                             png = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/TMBD/ico/new.png")
                        else:
                             png = LoadPixmap("/usr/lib/enigma2/python/Plugins/Extensions/TMBD/ico/wathc.png")
                res.append(MultiContentEntryProgress(pos=(25,9), size=(45,6), percent=dataext, borderWidth=1, foreColor=partcol, foreColorSelected=None, backColor=None, backColorSelected=None))  
                res.append((eListboxPythonMultiContent.TYPE_PIXMAP_ALPHATEST, 0, 1, 20, 20, png))
                res.append(MultiContentEntryText(pos=(75, 0), size=(width - 80, ih), font=0, flags=RT_HALIGN_LEFT, text=nametxt))

            elif self.descr_state == MovieList.SHOW_DESCRIPTION or not len:
    		if config.movielist.originalname.value:
                    filename = os.path.realpath(pathName)
                    nams = self.getarealname(filename)  		
                    if nams != "":
                            nametxt = nams
                    else:
                            nametxt = data.txt 
                else:
                    nametxt = data.txt            
                dateSize = ih * 145 / 25
                res.append(MultiContentEntryText(pos=(iconSize, 0), size=(width - iconSize - dateSize, ih), font=0, flags=RT_HALIGN_LEFT, text=nametxt))
                res.append(MultiContentEntryText(pos=(width - dateSize, 4), size=(dateSize, ih), font=1, flags=RT_HALIGN_RIGHT, text=begin_string))
            else:
    		if config.movielist.originalname.value:
                    filename = os.path.realpath(pathName)
                    nams = self.getarealname(filename)  		
                    if nams != "":
                            nametxt = nams
                    else:
                            nametxt = data.txt 
                else:
                    nametxt = data.txt                   
                lenSize = ih * 3
                res.append(MultiContentEntryText(pos=(iconSize, 0), size=(width - lenSize - iconSize, ih), font=0, flags=RT_HALIGN_LEFT, text=nametxt))
                res.append(MultiContentEntryText(pos=(width - lenSize, 0), size=(lenSize, ih), font=0, flags=RT_HALIGN_RIGHT, text=len))
            return res

    def getalttext(self, filename):
        name = ""
        if filename != "":
	        p = os.path.split(filename)
	        if not p[1]:
		        p = os.path.split(p)
                name = '%s' % (p[1])                  
	return name
    
    def getarealname(self, filename):
        nameext = ""
        namedesc = ""
        altname = ""
        namefil = ""
        namedir = ""
        nams = ""
        if config.plugins.tmbd.movieselectionparts.value == "none":
            nameext = filename[:-2] + 'txt'
            if fileExists(nameext):
                namedesc = nameext
            else:
                if filename != "":
                    altname = os.path.split(filename)[0]
                    namedesc = altname + '.txt'
                    namefil = self.getalttext(altname)
        else:
            altext, namedir = os.path.split(filename)
            nameext = config.plugins.tmbd.movieselectionparts.value + "/" + namedir[:-2] + 'txt'
        if fileExists(nameext):
            f = open(nameext)
            for line in f:
                if line.find('Origname:') > -1:
                    nams = line.split('Origname:')[1].strip()
                else:
                    nams = ""                            
            f.close()
        return nams

    def moveToFirstMovie(self):
        if self.firstFileEntry < len(self.list):
            self.instance.moveSelectionTo(self.firstFileEntry)
        else:
            self.moveToFirst()

    def moveToParentDirectory(self):
        if self.parentDirectory < len(self.list):
            self.instance.moveSelectionTo(self.parentDirectory)
        else:
            self.moveToFirst()

    def moveToLast(self):
        if self.list:
            self.instance.moveSelectionTo(len(self.list) - 1)

    def moveToFirst(self):
        if self.list:
            self.instance.moveSelectionTo(0)

    def moveToIndex(self, index):
        self.instance.moveSelectionTo(index)

    def getCurrentIndex(self):
        return self.instance.getCurrentIndex()

    def getCurrentEvent(self):
        l = self.l.getCurrentSelection()
        return l and l[0] and l[1] and l[1].getEvent(l[0])

    def getCurrent(self):
        l = self.l.getCurrentSelection()
        return l and l[0]

    def getItem(self, index):
        if self.list:
            if len(self.list) > index:
                return self.list[index] and self.list[index][0]

    GUI_WIDGET = eListbox

    def postWidgetCreate(self, instance):
        instance.setContent(self.l)
        instance.selectionChanged.get().append(self.selectionChanged)

    def preWidgetRemove(self, instance):
        instance.setContent(None)
        instance.selectionChanged.get().remove(self.selectionChanged)
        return

    def reload(self, root = None, filter_tags = None):
        if self.reloadDelayTimer is not None:
            self.reloadDelayTimer.stop()
            self.reloadDelayTimer = None
        if root is not None:
            self.load(root, filter_tags)
        else:
            self.load(self.root, filter_tags)
        self.l.setList(self.list)
        return

    def removeService(self, service):
        for index, l in enumerate(self.list):
            if l[0] == service:
                del self.list[index]
                break

        self.l.setList(self.list)

    def __len__(self):
        return len(self.list)

    def __getitem__(self, index):
        return self.list[index]

    def __iter__(self):
        return self.list.__iter__()

    def load(self, root, filter_tags):
        del self.list[:]
        serviceHandler = eServiceCenter.getInstance()
        numberOfDirs = 0
        reflist = serviceHandler.list(root)
        if reflist is None:
            print 'listing of movies failed'
            return
        else:
            realtags = set()
            tags = {}
            rootPath = os.path.normpath(root.getPath())
            parent = None
            if len(rootPath) > 1 and os.path.realpath(rootPath) != config.movielist.root.value:
                parent = os.path.split(os.path.normpath(rootPath))[0]
                if parent and parent not in defaultInhibitDirs:
                    if not parent.endswith('/'):
                        parent += '/'
                    ref = eServiceReference('2:0:1:0:0:0:0:0:0:0:' + parent)
                    ref.flags = eServiceReference.flagDirectory
                    self.list.append((ref,
                     None,
                     0,
                     -1))
                    numberOfDirs += 1
            while 1:
                serviceref = reflist.getNext()
                if not serviceref.valid():
                    break
                info = serviceHandler.info(serviceref)
                if info is None:
                    info = justStubInfo
                begin = info.getInfo(serviceref, iServiceInformation.sTimeCreate)
                if serviceref.flags & eServiceReference.mustDescent:
                    self.list.append((serviceref,
                     info,
                     begin,
                     -1))
                    numberOfDirs += 1
                    continue
                this_tags = info.getInfoString(serviceref, iServiceInformation.sTags).split(' ')
                name = info.getName(serviceref)
                if this_tags == ['']:
                    this_tags = name.replace(',', ' ').replace('.', ' ').split()
                else:
                    realtags.update(this_tags)
                for tag in this_tags:
                    if tags.has_key(tag):
                        tags[tag].append(name)
                    else:
                        tags[tag] = [name]

                if filter_tags is not None:
                    this_tags = set(this_tags)
                    if not this_tags.issuperset(filter_tags):
                        print 'Skipping', name, 'tags=', this_tags, ' filter=', filter_tags
                        continue
                self.list.append((serviceref,
                 info,
                 begin,
                 -1))

            self.firstFileEntry = numberOfDirs
            self.parentDirectory = 0
            if self.sort_type == MovieList.SORT_ALPHANUMERIC:
                self.list.sort(key=self.buildAlphaNumericSortKey)
            elif self.sort_type == MovieList.SORT_ALPHANUMERIC_FLAT:
                self.list.sort(key=self.buildAlphaNumericFlatSortKey)
            elif self.sort_type == MovieList.SORT_ALPHANUMERIC_FLAT_REVERSE:
                self.list.sort(key=self.buildAlphaNumericFlatSortKey, reverse=True)
            else:
                self.list.sort(key=self.buildBeginTimeSortKey)
            if self.sort_type == MovieList.SHUFFLE:
                dirlist = self.list[:numberOfDirs]
                shufflelist = self.list[numberOfDirs:]
                random.shuffle(shufflelist)
                self.list = dirlist + shufflelist
            elif self.sort_type == MovieList.SORT_ALPHANUMERIC_REVERSE:
                self.list = self.list[:numberOfDirs] + sorted(self.list[numberOfDirs:], key=self.buildAlphaNumericSortKey, reverse=True)
            elif self.sort_type == MovieList.SORT_RECORDED_REVERSE:
                self.list = self.list[:numberOfDirs] + sorted(self.list[numberOfDirs:], key=self.buildBeginTimeSortKey, reverse=True)
            if self.root and numberOfDirs > 0:
                rootPath = os.path.normpath(self.root.getPath())
                if not rootPath.endswith('/'):
                    rootPath += '/'
                if rootPath != parent:
                    for index, item in enumerate(self.list):
                        if item[0].flags & eServiceReference.mustDescent:
                            itempath = os.path.normpath(item[0].getPath())
                            if not itempath.endswith('/'):
                                itempath += '/'
                            if itempath == rootPath:
                                self.parentDirectory = index
                                break

            self.root = root
            rtags = {}
            for tag, movies in tags.items():
                if len(movies) > 1 or tag in realtags:
                    movies = tuple(movies)
                    item = rtags.get(movies, [])
                    if not item:
                        rtags[movies] = item
                    item.append(tag)

            self.tags = {}
            for movies, tags in rtags.items():
                movie = movies[0]
                tags.sort(key=movie.find)
                first = movie.find(tags[0])
                last = movie.find(tags[-1]) + len(tags[-1])
                match = movie
                start = 0
                end = len(movie)
                for m in movies[1:]:
                    if m[start:end] != match:
                        if not m.startswith(movie[:last]):
                            start = first
                        if not m.endswith(movie[first:]):
                            end = last
                        match = movie[start:end]
                        if m[start:end] != match:
                            match = ''
                            break

                if match:
                    self.tags[match] = set(tags)
                    continue
                else:
                    match = ' '.join(tags)
                    if len(match) > 2:
                        self.tags[match] = set(tags)

            return

    def buildAlphaNumericSortKey(self, x):
        ref = x[0]
        name = x[1] and x[1].getName(ref)
        if ref.flags & eServiceReference.mustDescent:
            return (0, name and name.lower() or '', -x[2])
        return (1, name and name.lower() or '', -x[2])

    def buildAlphaNumericFlatSortKey(self, x):
        ref = x[0]
        name = x[1] and x[1].getName(ref)
        if name and ref.flags & eServiceReference.mustDescent:
            p = os.path.split(name)
            if not p[1]:
                p = os.path.split(p[0])
            name = p[1]
        return (1, name and name.lower() or '', -x[2])

    def buildBeginTimeSortKey(self, x):
        ref = x[0]
        if ref.flags & eServiceReference.mustDescent:
            return (0, x[1] and x[1].getName(ref).lower() or '')
        return (1, -x[2])

    def moveTo(self, serviceref):
        count = 0
        for x in self.list:
            if x[0] == serviceref:
                self.instance.moveSelectionTo(count)
                return True
            count += 1

        return False

    def moveDown(self):
        self.instance.moveSelection(self.instance.moveDown)

    def moveUp(self):
        self.instance.moveSelection(self.instance.moveUp)

    def moveToChar(self, char, lbl = None):
        self._char = char
        self._lbl = lbl
        if lbl:
            lbl.setText(self._char)
            lbl.visible = True
        self.moveToCharTimer = eTimer()
        self.moveToCharTimer.callback.append(self._moveToChrStr)
        self.moveToCharTimer.start(1000, True)

    def moveToString(self, char, lbl = None):
        self._char = self._char + char.upper()
        self._lbl = lbl
        if lbl:
            lbl.setText(self._char)
            lbl.visible = True
        self.moveToCharTimer = eTimer()
        self.moveToCharTimer.callback.append(self._moveToChrStr)
        self.moveToCharTimer.start(1000, True)

    def _moveToChrStr(self):
        currentIndex = self.instance.getCurrentIndex()
        found = False
        if currentIndex < len(self.list) - 1:
            itemsBelow = self.list[currentIndex + 1:]
            for index, item in enumerate(itemsBelow):
                ref = item[0]
                itemName = getShortName(item[1].getName(ref).upper(), ref)
                if len(self._char) == 1 and itemName.startswith(self._char):
                    found = True
                    self.instance.moveSelectionTo(index + currentIndex + 1)
                    break
                elif len(self._char) > 1 and itemName.find(self._char) >= 0:
                    found = True
                    self.instance.moveSelectionTo(index + currentIndex + 1)
                    break

        if found == False and currentIndex > 0:
            itemsAbove = self.list[1:currentIndex]
            for index, item in enumerate(itemsAbove):
                ref = item[0]
                itemName = getShortName(item[1].getName(ref).upper(), ref)
                if len(self._char) == 1 and itemName.startswith(self._char):
                    found = True
                    self.instance.moveSelectionTo(index + 1)
                    break
                elif len(self._char) > 1 and itemName.find(self._char) >= 0:
                    found = True
                    self.instance.moveSelectionTo(index + 1)
                    break

        self._char = ''
        if self._lbl:
            self._lbl.visible = False


def getShortName(name, serviceref):
    if serviceref.flags & eServiceReference.mustDescent:
        pathName = serviceref.getPath()
        p = os.path.split(pathName)
        if not p[1]:
            p = os.path.split(p[0])
        return p[1].upper()
    else:
        return name
